#![deny(unsafe_op_in_unsafe_fn)]
#![feature(sync_unsafe_cell)]
#![feature(generic_const_exprs)]

use std::ops::Range;

use jni::{
  sys::{jint, JNI_VERSION_1_8},
  JNIEnv,
};

pub mod component;
mod computer;
mod core;
pub mod unsync_cell;
mod memory_map;

#[macro_export]
macro_rules! assert_positive {
  ($env: expr, $value: expr) => {{
    let value = $value;
    if value < 0 {
      ($env)
        .throw(format!(
          "input {} from java cannot be negative! got {value}",
          stringify!($value)
        ))
        .unwrap();
      return;
    }
    value as _
  }};
  ($env: expr, $value: expr, $err: expr) => {{
    let value = $value;
    if value < 0 {
      ($env)
        .throw(format!(
          "input {} from java cannot be negative! got {value}",
          stringify!($value)
        ))
        .unwrap();
      return $err;
    }
    value as _
  }};
}

#[macro_export]
macro_rules! range_of_field {
  ($type: ty, $field: ident) => {{
    const RANGE: std::ops::Range<usize> = {
      let size = {
        let v: std::mem::MaybeUninit<$type> = std::mem::MaybeUninit::uninit();
        size_of_val(unsafe { &v.assume_init_ref().$field })
      };

      let offset = std::mem::offset_of!($type, $field);

      offset as _..(offset + size) as _
    };

    let range: std::ops::Range<_> = RANGE.start as _..RANGE.end as _;
    range
  }};
}

fn overlapping<Idx: PartialOrd>(a: &Range<Idx>, b: &Range<Idx>) -> bool {
  a.contains(&b.start) && a.contains(&b.end)
}

fn subtract_range(a: &Range<usize>, b: &Range<usize>) -> Range<usize> {
  a.start - b.start..a.end - b.end
}

pub fn align_up(value: u64, alignment: u64) -> u64 {
  (value + alignment - 1) & !(alignment - 1)
}

struct UnicornWrap {

}

#[unsafe(no_mangle)]
pub extern "system" fn JNI_OnLoad<'local>(_: JNIEnv<'local>, _: usize) -> jint {
  JNI_VERSION_1_8
}
