package ca.sanae.golemcomputers.computer

import ca.sanae.golemcomputers.computer.RustNative.Companion.cleaner
import ca.sanae.golemcomputers.computer.components.ChatComponent
import ca.sanae.golemcomputers.computer.components.Component
import net.minecraft.world.World
import java.nio.ByteBuffer
import java.nio.channels.FileChannel
import java.nio.file.Files
import kotlin.io.path.Path

class Computer(val world: World) {
  @Suppress("unused")
  private val address: Long = 0

  init {
    new(1, 0x1000, intArrayOf(0x10000))
    cleaner.register(this) {
      free()
    }
    val path = Path("/home/aubrey/Projects/golem-software/assembly.bin")
    val channel = FileChannel.open(path);
    val buffer = channel.map(FileChannel.MapMode.READ_ONLY, 0, channel.size())
    val wakeAddress = buffer.getUnsignedInt();

    updateRomTest(buffer.rewind())
    println("waking at 0x${wakeAddress.toString(16)}")
    insertComponent(0, ChatComponent(this))
    wakeCoreAt(0, wakeAddress)
  }

  fun ByteBuffer.getUnsignedInt(): Long {
    return Integer.reverseBytes(this.getInt()).toLong() and 0xFFFFFFFFL
  }

  fun insertComponent(coreIndex: Int, component: Component) {
    insertComponentNative(
      coreIndex,
      component,
      component.info.typeName.toString(),
      component.info.protocol.toString(),
      component.info.bytesPerTick,
      component.info.version
    )
  }

  private external fun new(componentCapacity: Int, memorySize: Int, coreIps: IntArray): Long
  private external fun free()
  private external fun updateRomTest(data: ByteBuffer)
  private external fun wakeCoreAt(coreIndex: Int, address: Long)
  private external fun insertComponentNative(
    index: Int,
    component: Component,
    typeName: String,
    protocol: String,
    bytesPerTick: UInt,
    version: UInt
  )

  external fun tick();

}