package ca.sanae.golemcomputers.computer.components

import com.github.pbbl.heap.ByteBufferPool
import com.jogamp.common.util.LFRingbuffer
import com.jogamp.common.util.Ringbuffer
import net.minecraft.util.Identifier
import java.nio.ByteBuffer

abstract class Component(val info: ComponentInfo) {
  @Suppress("unused")
  private val address: Long = 0
  private val pool = ByteBufferPool()
  abstract fun gotByteChunk(buffer: ByteBuffer)
  private external fun writeBytesNative(buffer: ByteBuffer): Boolean
  fun writeBytes(lambda: ((size: Int) -> ByteBuffer) -> ByteBuffer) {
    synchronized(pool) {
      val buffer = lambda(pool::take)
      val overflowed = writeBytesNative(buffer)
      pool.give(buffer)
      if (overflowed) {
        overflowShutdown()
      }
    }
  }

  // called when the component runs out of runway to store unread data
  open fun overflowShutdown() {}
  abstract fun restarted()

  data class ComponentInfo(val typeName: Identifier, val protocol: Identifier, val bytesPerTick: UInt, val version: UInt)
}